"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirebaseAuth = void 0;
const app_1 = __importDefault(require("firebase/app"));
const entropic_bond_1 = require("entropic-bond");
const firebase_helper_1 = require("../firebase-helper");
const utils_1 = require("../utils/utils");
class FirebaseAuth extends entropic_bond_1.AuthService {
    constructor() {
        super();
        this.registerCredentialProviders();
    }
    signUp(signData) {
        const { authProvider, verificationLink } = signData;
        if (authProvider.slice(0, 5) === 'email') {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const credentialFactory = this.credentialProviders['email-sign-up'];
                    const userCredentials = yield credentialFactory(signData);
                    if (signData.name) {
                        userCredentials.user.updateProfile({
                            displayName: signData.name
                        });
                    }
                    if (verificationLink) {
                        yield userCredentials.user.sendEmailVerification({
                            url: verificationLink
                        });
                    }
                    resolve(yield this.toUserCredentials(userCredentials.user));
                }
                catch (error) {
                    reject({
                        code: utils_1.camelCase(error.code.slice(5)),
                        message: error.message
                    });
                }
            }));
        }
        else
            return this.login(signData);
    }
    login(signData) {
        const { authProvider } = signData;
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                const credentialFactory = this.credentialProviders[authProvider];
                const userCredentials = yield credentialFactory(signData);
                resolve(yield this.toUserCredentials(userCredentials.user));
            }
            catch (error) {
                reject({
                    code: utils_1.camelCase(error.code.slice(5)),
                    message: error.message
                });
            }
        }));
    }
    logout() {
        return firebase_helper_1.FirebaseHelper.instance.auth().signOut();
    }
    onAuthStateChange(onChange) {
        firebase_helper_1.FirebaseHelper.instance.auth().onAuthStateChanged((credentials) => __awaiter(this, void 0, void 0, function* () {
            onChange(yield this.toUserCredentials(credentials));
        }));
    }
    toUserCredentials(nativeUserCredential) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!nativeUserCredential)
                return null;
            const claims = (yield nativeUserCredential.getIdTokenResult()).claims;
            return FirebaseAuth.convertCredentials(nativeUserCredential, claims);
        });
    }
    static convertCredentials(nativeUserCredential, claims) {
        return ({
            id: nativeUserCredential.uid,
            email: nativeUserCredential.email,
            name: nativeUserCredential.displayName,
            pictureUrl: nativeUserCredential.photoURL,
            phoneNumber: nativeUserCredential.phoneNumber,
            emailVerified: nativeUserCredential.emailVerified,
            customData: {
                planExpireDate: claims.planExpireDate,
                subscriptionPlan: claims.subscriptionPlan,
                gdprConsent: claims.gdprConsent
            },
            lastLogin: Date.now(),
            creationDate: new Date(nativeUserCredential.metadata.creationTime).getTime()
        });
    }
    registerCredentialProviders() {
        this.registerCredentialProvider('email-sign-up', signData => firebase_helper_1.FirebaseHelper.instance.auth()
            .createUserWithEmailAndPassword(signData.email, signData.password));
        this.registerCredentialProvider('email', signData => firebase_helper_1.FirebaseHelper.instance.auth()
            .signInWithEmailAndPassword(signData.email, signData.password));
        this.registerCredentialProvider('google', () => firebase_helper_1.FirebaseHelper.instance.auth()
            .signInWithPopup(new app_1.default.auth.GoogleAuthProvider()));
        this.registerCredentialProvider('facebook', () => firebase_helper_1.FirebaseHelper.instance.auth()
            .signInWithPopup(new app_1.default.auth.FacebookAuthProvider()));
        this.registerCredentialProvider('anonymous', () => __awaiter(this, void 0, void 0, function* () {
            return firebase_helper_1.FirebaseHelper.instance.auth()
                .signInAnonymously();
        }));
    }
}
exports.FirebaseAuth = FirebaseAuth;
//# sourceMappingURL=firebase-auth.js.map