"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
global.XMLHttpRequest = require('xhr2');
const node_fetch_1 = __importDefault(require("node-fetch"));
const firebase_cloud_storage_1 = require("./firebase-cloud-storage");
const firebase_helper_1 = require("../firebase-helper");
const firebase_datasource_1 = require("../store/firebase-datasource");
const entropic_bond_1 = require("entropic-bond");
// Note about tests leaking. I've been checking and looks like firebase.storage 
// methods are the responsible for the test leaking (as firebase v. 8.6.3).
class File {
}
global.File = File;
let Test = class Test extends entropic_bond_1.Persistent {
    constructor() {
        super(...arguments);
        this._file = new entropic_bond_1.StoredFile();
    }
    get file() {
        return this._file;
    }
};
__decorate([
    entropic_bond_1.persistent
], Test.prototype, "_file", void 0);
Test = __decorate([
    entropic_bond_1.registerPersistentClass('Test')
], Test);
describe('Firebase Cloud Storage', () => {
    const blobData1 = new Uint8Array([0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x21]);
    const blobData2 = new Uint8Array([0x6c, 0x6c, 0x6f, 0x2c, 0x48, 0x65, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x21]);
    let file;
    firebase_helper_1.FirebaseHelper.setFirebaseConfig({
        projectId: 'demo-test',
        storageBucket: 'default-bucket'
    });
    firebase_helper_1.FirebaseHelper.useEmulator({ firestorePort: 9080 });
    entropic_bond_1.Store.useDataSource(new firebase_datasource_1.FirebaseDatasource());
    beforeEach(() => {
        entropic_bond_1.CloudStorage.useCloudStorage(new firebase_cloud_storage_1.FirebaseCloudStorage());
        file = new entropic_bond_1.StoredFile();
    });
    it('should save and get a url', () => __awaiter(void 0, void 0, void 0, function* () {
        yield file.store(blobData1);
        expect(file.url).toContain(file.id);
    }));
    it('should report metadata', () => __awaiter(void 0, void 0, void 0, function* () {
        yield file.store(blobData1, 'test.dat');
        expect(file.originalFileName).toEqual('test.dat');
        expect(file.provider.className).toEqual('FirebaseCloudStorage');
    }));
    it('should delete file', () => __awaiter(void 0, void 0, void 0, function* () {
        yield file.store(blobData1);
        yield file.delete();
        expect(file.url).not.toBeDefined();
    }));
    it('should overwrite file on subsequent writes', () => __awaiter(void 0, void 0, void 0, function* () {
        yield file.store(blobData1);
        const firstUrl = file.url;
        let resp = yield node_fetch_1.default(file.url);
        expect(yield resp.text()).toEqual('Hello, world!');
        yield file.store(blobData2);
        resp = yield node_fetch_1.default(file.url);
        expect(file.url.slice(0, file.url.indexOf('token'))).toEqual(firstUrl.slice(0, firstUrl.indexOf('token')));
        expect(yield resp.text()).toEqual('llo,He world!');
    }));
    it('should trigger events', done => {
        const cb = jest.fn();
        file.store(blobData1).then(() => {
            expect(cb).toHaveBeenCalledTimes(2);
            done();
        });
        file.uploadControl().onProgress(cb);
    });
    describe('Streaming', () => {
        let model;
        let testObj;
        beforeEach(() => {
            testObj = new Test();
            model = entropic_bond_1.Store.getModel(testObj);
        });
        it('should load object with StoredFile', () => __awaiter(void 0, void 0, void 0, function* () {
            yield testObj.file.store(blobData1, 'test.dat');
            yield model.save(testObj);
            const newTestObj = yield model.findById(testObj.id);
            expect(newTestObj.file).toBeInstanceOf(entropic_bond_1.StoredFile);
            expect(newTestObj.file.url).toContain(testObj.file.id);
        }));
        it('should replace file on save after load', () => __awaiter(void 0, void 0, void 0, function* () {
            const deleteSpy = jest.spyOn(testObj.file, 'delete');
            yield testObj.file.store(blobData1, 'test.dat');
            yield model.save(testObj);
            const newTestObj = yield model.findById(testObj.id);
            expect(newTestObj.file).toBeInstanceOf(entropic_bond_1.StoredFile);
            expect(newTestObj.file.url).toContain(testObj.file.id);
            expect(deleteSpy).not.toHaveBeenCalled();
            testObj.file.setDataToStore(blobData2);
            yield testObj.file.store();
            expect(deleteSpy).toHaveBeenCalled();
        }));
    });
});
//# sourceMappingURL=firebase-cloud-storage.spec.js.map