"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirebaseDatasource = void 0;
const entropic_bond_1 = require("entropic-bond");
const firebase_helper_1 = require("../firebase-helper");
class FirebaseDatasource extends entropic_bond_1.DataSource {
    constructor(emulator) {
        super();
        if (emulator)
            firebase_helper_1.FirebaseHelper.useEmulator(emulator);
        const { emulate, host, firestorePort } = firebase_helper_1.FirebaseHelper.emulator;
        if (emulate) {
            firebase_helper_1.FirebaseHelper.instance.firestore().useEmulator(host, firestorePort);
        }
    }
    findById(id, collectionName) {
        const db = firebase_helper_1.FirebaseHelper.instance.firestore();
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            try {
                const doc = yield db.collection(collectionName).doc(id).get();
                resolve(doc.data());
            }
            catch (error) {
                console.log(error);
                return null;
            }
        }));
    }
    save(collections) {
        const db = firebase_helper_1.FirebaseHelper.instance.firestore();
        const batch = db.batch();
        Object.entries(collections).forEach(([collectionName, collection]) => {
            collection.forEach(document => {
                const ref = db.collection(collectionName).doc(document.id);
                batch.set(ref, document);
            });
        });
        return batch.commit();
    }
    find(queryObject, collectionName) {
        let query = firebase_helper_1.FirebaseHelper.instance.firestore().collection(collectionName);
        entropic_bond_1.DataSource.toPropertyPathOperations(queryObject.operations).forEach(operation => {
            query = query.where(operation.property, operation.operator, operation.value);
        });
        if (queryObject.sort) {
            query = query.orderBy(queryObject.sort.propertyName, queryObject.sort.order);
        }
        this._lastQuery = query;
        if (queryObject.limit) {
            this._lastLimit = queryObject.limit;
            query = query.limit(queryObject.limit);
        }
        return this.getFromQuery(query);
    }
    delete(id, collectionName) {
        const db = firebase_helper_1.FirebaseHelper.instance.firestore();
        return db.collection(collectionName).doc(id).delete();
    }
    next(limit) {
        if (!this._lastQuery)
            throw new Error('You should perform a query prior to using method next');
        this._lastLimit = limit || this._lastLimit;
        let query = this._lastQuery
            .limit(this._lastLimit)
            .startAfter(this._lastDocRetrieved);
        return this.getFromQuery(query);
    }
    // prev should be used with next in reverse order
    // prev( limit?: number ): Promise< DocumentObject[] > {
    // }
    getFromQuery(query) {
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            const doc = yield query.get();
            this._lastDocRetrieved = doc.docs[doc.docs.length - 1];
            resolve(doc.docs.map(doc => doc.data()));
        }));
    }
}
exports.FirebaseDatasource = FirebaseDatasource;
//# sourceMappingURL=firebase-datasource.js.map