"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
describe('Utils', () => {
    describe('replaceValues', () => {
        const text = 'The population of ${country} is ${ people} million \
											and the GDP is $${ gdpValue } million';
        const expected = 'The population of U.S.A. is 100 million \
											and the GDP is $1000 million';
        const vars = {
            country: 'U.S.A.',
            people: '100',
            gdpValue: '1000'
        };
        it('should replace vars as a template literal', () => {
            expect(utils_1.replaceValue(text, vars)).toEqual(expected);
        });
        it('should replace vars with empty value', () => {
            vars.country = '';
            vars.people = undefined;
            expect(utils_1.replaceValue('The population of ${country} is ${ people} million', vars)).toEqual('The population of  is  million');
        });
        it('should resturn empty string on falsy', () => {
            expect(utils_1.replaceValue(undefined, vars)).toEqual('');
            expect(utils_1.replaceValue(null, vars)).toEqual('');
            expect(utils_1.replaceValue('', vars)).toEqual('');
        });
    });
    describe('Camel/Snake case', () => {
        it('should convert to camel case', () => {
            expect(utils_1.camelCase('snake_case')).toEqual('snakeCase');
            expect(utils_1.camelCase('snake-case')).toEqual('snakeCase');
            expect(utils_1.camelCase('snake-Case')).toEqual('snakeCase');
            expect(utils_1.camelCase('snake_Case')).toEqual('snakeCase');
            expect(utils_1.camelCase('Snake-Case')).toEqual('SnakeCase');
            expect(utils_1.camelCase('Snake_Case')).toEqual('SnakeCase');
        });
        it('should convert to snake case', () => {
            expect(utils_1.snakeCase('snakeCase')).toEqual('snake-case');
            expect(utils_1.snakeCase('snakeCase', '_')).toEqual('snake_case');
            expect(utils_1.snakeCase('SnakeCase')).toEqual('snake-case');
            expect(utils_1.snakeCase('SnakeCase', '_')).toEqual('snake_case');
        });
    });
});
//# sourceMappingURL=utils.spec.js.map